// ----------------------------------------------------------------------------
// * Raima Database Manager
// *
// * Copyright (c) 2012 Raima Inc.,  All rights reserved.
// *
// * Use of this software, whether in source code format, or in executable,
// * binary object code form, is governed by the Raima LICENSE which
// * is fully described in the LICENSE.TXT file, included within this
// * distribution of files.
// * ----------------------------------------------------------------------------

#include <iostream>

// rdm.h is required in all files using the RDM Database
#include "rdm.h"

// obj09_db.h was generated from the DDL file obj09_db.ddl.
#include "obj09_db.h"

// obj09_db_api.h was generated from the DDL file obj09_db.ddl. with 
// the -cpp option
#include "obj09_db_api.h"

// Name space for the RDM C++ interface
using namespace RDM_CPP;

// EXAMPLE - obj09 for C++
// Each time you run this example:
//   1) The database is initialized (all existing contents removed)
//   2) Several new offices are added to the NA database
//   3) Several new offices are added to the EMEA database
//   4) Data is queried from the NA office
//   5) Data is queried from the EMEA office
//   6) Data is queried from from both databases as a union (unified view
//      of two identically structured databases)

// The DDL (Database Definition Language) is in obj09_db.ddl. The database
// schema is embedded into the application and the database will be created
// the first time the application is run.  Subsequent runs will use the now
// existing database.

// The following RDM C++ methods are illustrated:
//      Db->Open
//      Db->Initialize
//      Db->New_recordWithFieldValues
//      Db->Get_recordsByKey
//      Cursor->AfterLast
//      Cursor->GetFieldValues
//      Cursor++
// They are fully documented in http://docs.raima.com/rdm/.

// Each of these methods can throw and an exception of type rdm_exception

// The actual database is stored in a directory named 'obj09_db' in this
// project directory.

class obj09
{
public:
    Db_obj09_db db_na;
    Db_obj09_db db_emea;
    Db_obj09_db db_combined;

    void InsertData () throw (rdm_exception&);
    void DisplayOffices (Db_obj09_db&) throw (rdm_exception&);
    void Initialize () throw (rdm_exception&);
    void Cleanup () throw (rdm_exception&);
};

void obj09::Initialize()
{
    // Open the North American database and initialize it
    try
    {
        db_na = db_na.Open("na_obj09_db");
        db_na.Initialize();
    }
    catch (rdm_exception& e)
    {
        cerr << "Error " << e.error() << " (" << e.what() << ") opening/initializing na_obj09_db database" << endl;
        throw (e);
        return;
    }

    // Open the EMEA database and initialize it
    try
    {
        db_emea = db_emea.Open("emea_obj09_db");
        db_emea.Initialize();
    }
    catch (rdm_exception& e)
    {
        cerr << "Error " << e.error() << " (" << e.what() << ") opening/initializing emea_obj09_db database" << endl;
        throw (e);
        return;
    }

    // Open the database union
    try
    {
        db_combined = db_emea.Open("na_obj09_db|emea_obj09_db", "r");
    }
    catch (rdm_exception& e)
    {
        cerr << "Error " << e.error() << " (" << e.what() << ") opening obj09_db database union" << endl;
        throw (e);
        return;
    }
    return;
}

void obj09::InsertData()
{
    struct office office_rec;    
    
    try
    {
        // Create offices in the NA database
        strcpy(office_rec.name, "Seattle");
        db_na.New_office_recordWithFieldValues(&office_rec);

        strcpy(office_rec.name, "Boise");
        db_na.New_office_recordWithFieldValues(&office_rec);

        strcpy(office_rec.name, "San Francisco");
        db_na.New_office_recordWithFieldValues(&office_rec);

        strcpy(office_rec.name, "Dallas");
        db_na.New_office_recordWithFieldValues(&office_rec);

        // Create offices in the EMEA database
        strcpy(office_rec.name, "Paris");
        db_emea.New_office_recordWithFieldValues(&office_rec);

        strcpy(office_rec.name, "London");
        db_emea.New_office_recordWithFieldValues(&office_rec);

        strcpy(office_rec.name, "Dublin");
        db_emea.New_office_recordWithFieldValues(&office_rec);

        strcpy(office_rec.name, "Zurich");
        db_emea.New_office_recordWithFieldValues(&office_rec);

        strcpy(office_rec.name, "Madrid");
        db_emea.New_office_recordWithFieldValues(&office_rec);
    }
    catch (rdm_exception& e)
    {
        cerr << "Error " << e.error() << " (" << e.what() << ") adding records" << endl;
        throw (e);
        return;
    }
    return;
}
    
void obj09::DisplayOffices(
    Db_obj09_db& db)
{
    struct office office_rec;
    Cursor_office office_cursor;
    recordType rtypes[]= {Cursor_office::Type (), Cursor::NoMoreTypes()};

    try
    {
        // Get a read lock on the office record
        db.BeginRead(rtypes);

        // Loop through each office record and display it
        office_cursor = db.Get_office_recordsBy_name();
        while (!office_cursor.AfterLast())
        {
            office_cursor.GetFieldValues(&office_rec);
            cout << '\t' << office_rec.name << endl;
            office_cursor++;
        }
    }
    catch (rdm_exception& e)
    {
        db.End();
        cerr << "Error " << e.error() << " (" << e.what() << ") reading records" << endl;
    }
    db.End();
}

void obj09::Cleanup()
{
    return;
}

int32_t main(void)
{
    obj09 obj;

    try
    {
        obj.Initialize();
        obj.InsertData();

        cout << endl << "North American Offices" << endl;
        obj.DisplayOffices(obj.db_na);
        
        cout << endl << "EMEA Offices" << endl;
        obj.DisplayOffices(obj.db_emea);

        cout << endl << "All Worldwide Offices" << endl;
        obj.DisplayOffices(obj.db_combined);

        obj.Cleanup();
    }
    catch (rdm_exception& e)
    {
        cerr << "Error " << e.error() << " (" << e.what() << ")" << endl;
        return EXIT_FAILURE;
    }
    
    cout << endl << "Press <Enter> to continue . . .";
    getchar();
    return EXIT_SUCCESS;
}
